/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */
#pragma ident "@(#)chkey.c	1.9	92/04/21 SMI"

/*
 * Copyright (C) 1986, Sun Microsystems, Inc.
 */

/*
 * Command to change one's public key in the public key database
 */
#include <stdio.h>
#include <rpc/rpc.h>
#include <rpc/key_prot.h>
#include <rpcsvc/ypclnt.h>
#include <pwd.h>
#include <string.h>

#define	index strchr
#define PK_FILES   1
#define PK_YP      2
#define PK_NISPLUS 3

extern char* get_password();
char program_name[256];

static char PKMAP[] = "publickey.byname";
static char PKFILE[] = "/etc/publickey";
#define MAXHOSTNAMELEN 256


main(argc, argv)
	int argc;
	char **argv;
{
	char netname[MAXNETNAMELEN+1];
	char domain[MAXNETNAMELEN+1];
	char public[HEXKEYBYTES + 1];
	char secret[HEXKEYBYTES + 1];
	char crypt1[HEXKEYBYTES + KEYCHECKSUMSIZE + 1];
	int status;
	char *pass;
	uid_t uid;
	int force = 0;
	int pk_database = 0;
	char *self;
	char* pk_service = NULL;
	char c;
	extern int optind;
	extern char *optarg;

	self = argv[0];
	strcpy( program_name, self );
	while ((c = getopt(argc, argv, "s:f")) != -1) {
		switch (c) {
		case 'f':
			force = 1;
			break;
		case 's':
			if( pk_service == NULL )
				pk_service = optarg;
			else
				usage(self);
			break;
			
		default:
			usage(self);
		}
	}

	if( optind < argc ) {
		usage(self);
	}

	if( (pk_database = get_pk_source( pk_service )) == 0 )
		usage(self);

	getdomainname( domain, MAXHOSTNAMELEN );
	if( getnetnameof(netname, uid = geteuid(), domain) < 0 ) {
		(void)fprintf(stderr,
			      "chkey: cannot generate netname for uid %d\n",
			      uid);
		exit(1);
	}

	(void)printf("Generating new key for '%s'.\n", netname);

	if( (pass = get_password( uid, 1, NULL, force)) == 0)
		exit(1);

	genkeys(public, secret, pass);
	memcpy(crypt1, secret, HEXKEYBYTES);
	memcpy(crypt1 + HEXKEYBYTES, secret, KEYCHECKSUMSIZE);
	crypt1[HEXKEYBYTES + KEYCHECKSUMSIZE] = 0;
	xencrypt(crypt1, pass);

	status = setpublicmap(netname, public, crypt1, pk_database );

	if (status != 0) {
		switch( pk_database ) {
		case PK_YP:
			(void)fprintf(stderr,
		      "%s: unable to update yp database (%u): %s\n",
				      self, status, yperr_string(status));
			break;
		case PK_FILES:
			(void)fprintf(stderr,
		      "%s: unable to update publickey database\n", self);
			break;
		case PK_NISPLUS:
			(void)fprintf(stderr,
		      "%s: unable to update nisplus database\n", self);
			break;
		default:
			fprintf(stderr, 
				"could not update unknown database: %d\n", 
				pk_database);
		}
			
		exit(1);
	}

	if (uid == 0) {
		/*
		 * Root users store their key in /etc/$ROOTKEY so
		 * that they can auto reboot without having to be
		 * around to type a password. Storing this in a file
		 * is rather dubious: it should really be in the EEPROM
		 * so it does not go over the net.
		 */
		write_rootkey(secret);
	}
	keylogin(netname, secret);
	(void)printf("Done.\n");
	exit(0);
}

usage(name)
	char *name;
{
	(void)fprintf(stderr, 
		      "usage: %s [-f] [-s nisplus | nis | files] \n", name);
	exit(1);
}


/*
 * Set the entry in the public key file
 */
setpublicmap(netname, public, secret, database)
	char *netname;
	char *public;
	char *secret;
	int database;
{
	char pkent[1024];
	char* master;
	char *domain = NULL;

	(void)sprintf(pkent, "%s:%s", public, secret);
	switch( database ) {
	case PK_YP:
		(void)yp_get_default_domain(&domain);
		if (yp_master(domain, PKMAP, &master) != 0) {
			(void)fprintf(stderr,
				      "can't find master of publickey database\n");
			exit(1);
		}
		printf("Sending key change request to %s...\n", master);
		return (yp_update(domain, PKMAP, YPOP_STORE,
				  netname, strlen(netname), pkent, 
				  strlen(pkent)));
	case PK_FILES:
		return (localupdate(netname, PKFILE, YPOP_STORE,
				    strlen(netname), netname, 
				    strlen(pkent), pkent));
	case PK_NISPLUS:
		return( nisplus_update( netname, public, secret ) );
	default:
		printf( "%s: unknown database type: %d\n", 
		       program_name, database );
	}
}

